@@ -166,7 +166,7 @@ static void* arm64_fetch_context_reg(CONTEXT* ctx, unsigned regno, unsigned* siz
 #ifdef __aarch64__
     switch (regno)
     {
-
+    case CV_ARM64_PSTATE: *size = sizeof(ctx->Cpsr); return &ctx->Cpsr;
     case CV_ARM64_X0 +  0: *size = sizeof(ctx->X0);  return &ctx->X0;
     case CV_ARM64_X0 +  1: *size = sizeof(ctx->X1);  return &ctx->X1;
     case CV_ARM64_X0 +  2: *size = sizeof(ctx->X2);  return &ctx->X2;
@@ -201,7 +201,6 @@ static void* arm64_fetch_context_reg(CONTEXT* ctx, unsigned regno, unsigned* siz
     case CV_ARM64_LR:     *size = sizeof(ctx->Lr);     return &ctx->Lr;
     case CV_ARM64_SP:     *size = sizeof(ctx->Sp);     return &ctx->Sp;
     case CV_ARM64_PC:     *size = sizeof(ctx->Pc);     return &ctx->Pc;
-    case CV_ARM64_PSTATE: *size = sizeof(ctx->PState); return &ctx->PState;
     }
 #endif
     FIXME("Unknown register %x\n", regno);
@@ -212,6 +211,7 @@ static const char* arm64_fetch_regname(unsigned regno)
 {
     switch (regno)
     {
+    case CV_ARM64_PSTATE:  return "cpsr";
     case CV_ARM64_X0 +  0: return "x0";
     case CV_ARM64_X0 +  1: return "x1";
     case CV_ARM64_X0 +  2: return "x2";
@@ -246,7 +246,6 @@ static const char* arm64_fetch_regname(unsigned regno)
     case CV_ARM64_LR:     return "lr";
     case CV_ARM64_SP:     return "sp";
     case CV_ARM64_PC:     return "pc";
-    case CV_ARM64_PSTATE: return "cpsr";
     }
     FIXME("Unknown register %x\n", regno);
     return NULL;
