@@ -24,12 +24,6 @@
 #include "config.h"
 #include "wine/port.h"
 
-#include "ntstatus.h"
-#define WIN32_NO_STATUS
-#include "dbghelp_private.h"
-
-#ifdef HAVE_MACH_O_LOADER_H
-
 #include <assert.h>
 #include <stdarg.h>
 #include <errno.h>
@@ -40,6 +34,16 @@
 # include <sys/mman.h>
 #endif
 
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
+#include "dbghelp_private.h"
+#include "winternl.h"
+#include "wine/library.h"
+#include "wine/debug.h"
+#include "image_private.h"
+
+#ifdef HAVE_MACH_O_LOADER_H
+
 #include <mach-o/fat.h>
 #include <mach-o/loader.h>
 #include <mach-o/nlist.h>
@@ -62,10 +66,6 @@ struct dyld_all_image_infos {
 };
 #endif
 
-#include "winternl.h"
-#include "wine/library.h"
-#include "wine/debug.h"
-
 #ifdef WORDS_BIGENDIAN
 #define swap_ulong_be_to_host(n) (n)
 #else
@@ -75,8 +75,26 @@ struct dyld_all_image_infos {
 WINE_DEFAULT_DEBUG_CHANNEL(dbghelp_macho);
 
 
+#ifdef _WIN64
+typedef struct segment_command_64   macho_segment_command;
+typedef struct nlist_64             macho_nlist;
+
+#define TARGET_CPU_TYPE         CPU_TYPE_X86_64
+#define TARGET_MH_MAGIC         MH_MAGIC_64
+#define TARGET_SEGMENT_COMMAND  LC_SEGMENT_64
+#else
+typedef struct segment_command      macho_segment_command;
+typedef struct nlist                macho_nlist;
+
+#define TARGET_CPU_TYPE         CPU_TYPE_X86
+#define TARGET_MH_MAGIC         MH_MAGIC
+#define TARGET_SEGMENT_COMMAND  LC_SEGMENT
+#endif
+
+
 struct macho_module_info
 {
+    struct image_file_map       file_map;
     unsigned long               load_addr;
     unsigned short              in_use : 1,
                                 is_loader : 1;
@@ -94,36 +112,7 @@ struct macho_info
     const WCHAR*                module_name;    /* OUT found module name (if MACHO_INFO_NAME is set) */
 };
 
-/* structure holding information while handling a Mach-O image */
-#define BITS_PER_ULONG (sizeof(ULONG) * 8)
-#define ULONGS_FOR_BITS(nbits) (((nbits) + BITS_PER_ULONG - 1) / BITS_PER_ULONG)
-struct macho_file_map
-{
-    /* A copy of the Mach-O header for an individual architecture. */
-    struct mach_header          mach_header;
-
-    /* The mapped load commands. */
-    const struct load_command*  load_commands;
-
-    /* The portion of the file which is this architecture.  mach_header was
-     * read from arch_offset. */
-    unsigned                    arch_offset;
-    unsigned                    arch_size;
-
-    /* The range of address space covered by all segments. */
-    size_t                      segs_start;
-    size_t                      segs_size;
-
-    /* Map of which sections contain code.  Sections are accessed using 1-based
-     * index.  Bit 0 of this bitset indicates if the bitset has been initialized. */
-    RTL_BITMAP                  sect_is_code;
-    ULONG                       sect_is_code_buff[ULONGS_FOR_BITS(MAX_SECT + 1)];
-
-    /* The file. */
-    int                         fd;
-};
-
-static void macho_unmap_file(struct macho_file_map* fmap);
+static void macho_unmap_file(struct image_file_map* fmap);
 
 /******************************************************************
  *              macho_calc_range
@@ -133,13 +122,13 @@ static void macho_unmap_file(struct macho_file_map* fmap);
  * that encompasses it.  For a fat binary, the architecture will
  * itself be offset within the file, so take that into account.
  */
-static void macho_calc_range(const struct macho_file_map* fmap, unsigned offset,
-                             unsigned len, unsigned* out_aligned_offset,
-                             unsigned* out_aligned_end, unsigned* out_aligned_len,
-                             unsigned* out_misalign)
+static void macho_calc_range(const struct macho_file_map* fmap, unsigned long offset,
+                             unsigned long len, unsigned long* out_aligned_offset,
+                             unsigned long* out_aligned_end, unsigned long* out_aligned_len,
+                             unsigned long* out_misalign)
 {
-    unsigned pagemask = sysconf( _SC_PAGESIZE ) - 1;
-    unsigned file_offset, misalign;
+    unsigned long pagemask = sysconf( _SC_PAGESIZE ) - 1;
+    unsigned long file_offset, misalign;
 
     file_offset = fmap->arch_offset + offset;
     misalign = file_offset & pagemask;
@@ -156,21 +145,24 @@ static void macho_calc_range(const struct macho_file_map* fmap, unsigned offset,
  *
  * Maps a range (offset, length in bytes) from a Mach-O file into memory
  */
-static const char* macho_map_range(const struct macho_file_map* fmap, unsigned offset, unsigned len)
+static const char* macho_map_range(const struct macho_file_map* fmap, unsigned long offset, unsigned long len,
+                                   const char** base)
 {
-    unsigned    misalign, aligned_offset, aligned_map_end, map_size;
-    const void* aligned_ptr;
+    unsigned long   misalign, aligned_offset, aligned_map_end, map_size;
+    const void*     aligned_ptr;
 
-    TRACE("(%p/%d, 0x%08x, 0x%08x)\n", fmap, fmap->fd, offset, len);
+    TRACE("(%p/%d, 0x%08lx, 0x%08lx)\n", fmap, fmap->fd, offset, len);
 
     macho_calc_range(fmap, offset, len, &aligned_offset, &aligned_map_end,
                      &map_size, &misalign);
 
     aligned_ptr = mmap(NULL, map_size, PROT_READ, MAP_PRIVATE, fmap->fd, aligned_offset);
 
-    TRACE("Mapped (0x%08x - 0x%08x) to %p\n", aligned_offset, aligned_map_end, aligned_ptr);
+    TRACE("Mapped (0x%08lx - 0x%08lx) to %p\n", aligned_offset, aligned_map_end, aligned_ptr);
 
-    if (aligned_ptr == MAP_FAILED) return MACHO_NO_MAP;
+    if (aligned_ptr == MAP_FAILED) return IMAGE_NO_MAP;
+    if (base)
+        *base = aligned_ptr;
     return (const char*)aligned_ptr + misalign;
 }
 
@@ -179,24 +171,30 @@ static const char* macho_map_range(const struct macho_file_map* fmap, unsigned o
  *
  * Unmaps a range (offset, length in bytes) of a Mach-O file from memory
  */
-static void macho_unmap_range(const void** mapped, const struct macho_file_map* fmap,
-                              unsigned offset, unsigned len)
+static void macho_unmap_range(const char** base, const void** mapped, const struct macho_file_map* fmap,
+                              unsigned long offset, unsigned long len)
 {
-    TRACE("(%p, %p/%d, 0x%08x, 0x%08x)\n", mapped, fmap, fmap->fd, offset, len);
+    TRACE("(%p, %p, %p/%d, 0x%08lx, 0x%08lx)\n", base, mapped, fmap, fmap->fd, offset, len);
 
-    if (mapped && *mapped != MACHO_NO_MAP)
+    if ((mapped && *mapped != IMAGE_NO_MAP) || (base && *base != IMAGE_NO_MAP))
     {
-        unsigned    misalign, aligned_offset, aligned_map_end, map_size;
-        void*       aligned_ptr;
+        unsigned long   misalign, aligned_offset, aligned_map_end, map_size;
+        void*           aligned_ptr;
 
         macho_calc_range(fmap, offset, len, &aligned_offset, &aligned_map_end,
                          &map_size, &misalign);
 
-        aligned_ptr = (char*)*mapped - misalign;
+        if (mapped)
+            aligned_ptr = (char*)*mapped - misalign;
+        else
+            aligned_ptr = (void*)*base;
         if (munmap(aligned_ptr, map_size) < 0)
             WARN("Couldn't unmap the range\n");
-        TRACE("Unmapped (0x%08x - 0x%08x) from %p - %p\n", aligned_offset, aligned_map_end, aligned_ptr, (char*)aligned_ptr + map_size);
-        *mapped = MACHO_NO_MAP;
+        TRACE("Unmapped (0x%08lx - 0x%08lx) from %p - %p\n", aligned_offset, aligned_map_end, aligned_ptr, (char*)aligned_ptr + map_size);
+        if (mapped)
+            *mapped = IMAGE_NO_MAP;
+        if (base)
+            *base = IMAGE_NO_MAP;
     }
 }
 
@@ -208,14 +206,14 @@ static void macho_unmap_range(const void** mapped, const struct macho_file_map*
  * the munmap doesn't fragment the mapping.
  */
 static BOOL macho_map_ranges(const struct macho_file_map* fmap,
-                             unsigned offset1, unsigned len1,
-                             unsigned offset2, unsigned len2,
+                             unsigned long offset1, unsigned long len1,
+                             unsigned long offset2, unsigned long len2,
                              const void** mapped1, const void** mapped2)
 {
-    unsigned aligned_offset1, aligned_map_end1;
-    unsigned aligned_offset2, aligned_map_end2;
+    unsigned long aligned_offset1, aligned_map_end1;
+    unsigned long aligned_offset2, aligned_map_end2;
 
-    TRACE("(%p/%d, 0x%08x, 0x%08x, 0x%08x, 0x%08x, %p, %p)\n", fmap, fmap->fd,
+    TRACE("(%p/%d, 0x%08lx, 0x%08lx, 0x%08lx, 0x%08lx, %p, %p)\n", fmap, fmap->fd,
             offset1, len1, offset2, len2, mapped1, mapped2);
 
     macho_calc_range(fmap, offset1, len1, &aligned_offset1, &aligned_map_end1, NULL, NULL);
@@ -223,33 +221,33 @@ static BOOL macho_map_ranges(const struct macho_file_map* fmap,
 
     if (aligned_map_end1 < aligned_offset2 || aligned_map_end2 < aligned_offset1)
     {
-        *mapped1 = macho_map_range(fmap, offset1, len1);
-        if (*mapped1 != MACHO_NO_MAP)
+        *mapped1 = macho_map_range(fmap, offset1, len1, NULL);
+        if (*mapped1 != IMAGE_NO_MAP)
         {
-            *mapped2 = macho_map_range(fmap, offset2, len2);
-            if (*mapped2 == MACHO_NO_MAP)
-                macho_unmap_range(mapped1, fmap, offset1, len1);
+            *mapped2 = macho_map_range(fmap, offset2, len2, NULL);
+            if (*mapped2 == IMAGE_NO_MAP)
+                macho_unmap_range(NULL, mapped1, fmap, offset1, len1);
         }
     }
     else
     {
         if (offset1 < offset2)
         {
-            *mapped1 = macho_map_range(fmap, offset1, offset2 + len2 - offset1);
-            if (*mapped1 != MACHO_NO_MAP)
+            *mapped1 = macho_map_range(fmap, offset1, offset2 + len2 - offset1, NULL);
+            if (*mapped1 != IMAGE_NO_MAP)
                 *mapped2 = (const char*)*mapped1 + offset2 - offset1;
         }
         else
         {
-            *mapped2 = macho_map_range(fmap, offset2, offset1 + len1 - offset2);
-            if (*mapped2 != MACHO_NO_MAP)
+            *mapped2 = macho_map_range(fmap, offset2, offset1 + len1 - offset2, NULL);
+            if (*mapped2 != IMAGE_NO_MAP)
                 *mapped1 = (const char*)*mapped2 + offset1 - offset2;
         }
     }
 
     TRACE(" => %p, %p\n", *mapped1, *mapped2);
 
-    return (*mapped1 != MACHO_NO_MAP) && (*mapped2 != MACHO_NO_MAP);
+    return (*mapped1 != IMAGE_NO_MAP) && (*mapped2 != IMAGE_NO_MAP);
 }
 
 /******************************************************************
@@ -260,14 +258,14 @@ static BOOL macho_map_ranges(const struct macho_file_map* fmap,
  * macho_map_ranges.
  */
 static void macho_unmap_ranges(const struct macho_file_map* fmap,
-                               unsigned offset1, unsigned len1,
-                               unsigned offset2, unsigned len2,
+                               unsigned long offset1, unsigned long len1,
+                               unsigned long offset2, unsigned long len2,
                                const void** mapped1, const void** mapped2)
 {
-    unsigned    aligned_offset1, aligned_map_end1;
-    unsigned    aligned_offset2, aligned_map_end2;
+    unsigned long   aligned_offset1, aligned_map_end1;
+    unsigned long   aligned_offset2, aligned_map_end2;
 
-    TRACE("(%p/%d, 0x%08x, 0x%08x, 0x%08x, 0x%08x, %p/%p, %p/%p)\n", fmap, fmap->fd,
+    TRACE("(%p/%d, 0x%08lx, 0x%08lx, 0x%08lx, 0x%08lx, %p/%p, %p/%p)\n", fmap, fmap->fd,
             offset1, len1, offset2, len2, mapped1, *mapped1, mapped2, *mapped2);
 
     macho_calc_range(fmap, offset1, len1, &aligned_offset1, &aligned_map_end1, NULL, NULL);
@@ -275,24 +273,108 @@ static void macho_unmap_ranges(const struct macho_file_map* fmap,
 
     if (aligned_map_end1 < aligned_offset2 || aligned_map_end2 < aligned_offset1)
     {
-        macho_unmap_range(mapped1, fmap, offset1, len1);
-        macho_unmap_range(mapped2, fmap, offset2, len2);
+        macho_unmap_range(NULL, mapped1, fmap, offset1, len1);
+        macho_unmap_range(NULL, mapped2, fmap, offset2, len2);
     }
     else
     {
         if (offset1 < offset2)
         {
-            macho_unmap_range(mapped1, fmap, offset1, offset2 + len2 - offset1);
-            *mapped2 = MACHO_NO_MAP;
+            macho_unmap_range(NULL, mapped1, fmap, offset1, offset2 + len2 - offset1);
+            *mapped2 = IMAGE_NO_MAP;
         }
         else
         {
-            macho_unmap_range(mapped2, fmap, offset2, offset1 + len1 - offset2);
-            *mapped1 = MACHO_NO_MAP;
+            macho_unmap_range(NULL, mapped2, fmap, offset2, offset1 + len1 - offset2);
+            *mapped1 = IMAGE_NO_MAP;
         }
     }
 }
 
+/******************************************************************
+ *              macho_find_section
+ */
+BOOL macho_find_section(struct image_file_map* ifm, const char* segname, const char* sectname, struct image_section_map* ism)
+{
+    struct macho_file_map* fmap;
+    unsigned i;
+    char tmp[sizeof(fmap->sect[0].section->sectname)];
+
+    /* Other parts of dbghelp use section names like ".eh_frame".  Mach-O uses
+       names like "__eh_frame".  Convert those. */
+    if (sectname[0] == '.')
+    {
+        lstrcpynA(tmp, "__", sizeof(tmp));
+        lstrcpynA(tmp + 2, sectname + 1, sizeof(tmp) - 2);
+        sectname = tmp;
+    }
+
+    fmap = &ifm->u.macho;
+    for (i = 0; i < fmap->num_sections; i++)
+    {
+        if (strcmp(fmap->sect[i].section->sectname, sectname) == 0 &&
+            (!segname || strcmp(fmap->sect[i].section->sectname, segname) == 0))
+        {
+            ism->fmap = ifm;
+            ism->sidx = i;
+            return TRUE;
+        }
+    }
+
+    ism->fmap = NULL;
+    ism->sidx = -1;
+    return FALSE;
+}
+
+/******************************************************************
+ *              macho_map_section
+ */
+const char* macho_map_section(struct image_section_map* ism)
+{
+    struct macho_file_map* fmap = &ism->fmap->u.macho;
+
+    assert(ism->fmap->modtype == DMT_MACHO);
+    if (ism->sidx < 0 || ism->sidx >= ism->fmap->u.macho.num_sections)
+        return IMAGE_NO_MAP;
+
+    return macho_map_range(fmap, fmap->sect[ism->sidx].section->offset, fmap->sect[ism->sidx].section->size,
+                           &fmap->sect[ism->sidx].mapped);
+}
+
+/******************************************************************
+ *              macho_unmap_section
+ */
+void macho_unmap_section(struct image_section_map* ism)
+{
+    struct macho_file_map* fmap = &ism->fmap->u.macho;
+
+    if (ism->sidx >= 0 && ism->sidx < fmap->num_sections && fmap->sect[ism->sidx].mapped != IMAGE_NO_MAP)
+    {
+        macho_unmap_range(&fmap->sect[ism->sidx].mapped, NULL, fmap, fmap->sect[ism->sidx].section->offset,
+                          fmap->sect[ism->sidx].section->size);
+    }
+}
+
+/******************************************************************
+ *              macho_get_map_rva
+ */
+DWORD_PTR macho_get_map_rva(const struct image_section_map* ism)
+{
+    if (ism->sidx < 0 || ism->sidx >= ism->fmap->u.macho.num_sections)
+        return 0;
+    return ism->fmap->u.macho.sect[ism->sidx].section->addr - ism->fmap->u.macho.segs_start;
+}
+
+/******************************************************************
+ *              macho_get_map_size
+ */
+unsigned macho_get_map_size(const struct image_section_map* ism)
+{
+    if (ism->sidx < 0 || ism->sidx >= ism->fmap->u.macho.num_sections)
+        return 0;
+    return ism->fmap->u.macho.sect[ism->sidx].section->size;
+}
+
 /******************************************************************
  *              macho_map_load_commands
  *
@@ -300,10 +382,10 @@ static void macho_unmap_ranges(const struct macho_file_map* fmap,
  */
 static const struct load_command* macho_map_load_commands(struct macho_file_map* fmap)
 {
-    if (fmap->load_commands == MACHO_NO_MAP)
+    if (fmap->load_commands == IMAGE_NO_MAP)
     {
         fmap->load_commands = (const struct load_command*) macho_map_range(
-                fmap, sizeof(fmap->mach_header), fmap->mach_header.sizeofcmds);
+                fmap, sizeof(fmap->mach_header), fmap->mach_header.sizeofcmds, NULL);
         TRACE("Mapped load commands: %p\n", fmap->load_commands);
     }
 
@@ -317,10 +399,10 @@ static const struct load_command* macho_map_load_commands(struct macho_file_map*
  */
 static void macho_unmap_load_commands(struct macho_file_map* fmap)
 {
-    if (fmap->load_commands != MACHO_NO_MAP)
+    if (fmap->load_commands != IMAGE_NO_MAP)
     {
         TRACE("Unmapping load commands: %p\n", fmap->load_commands);
-        macho_unmap_range((const void**)&fmap->load_commands, fmap,
+        macho_unmap_range(NULL, (const void**)&fmap->load_commands, fmap,
                     sizeof(fmap->mach_header), fmap->mach_header.sizeofcmds);
     }
 }
@@ -356,7 +438,7 @@ static int macho_enum_load_commands(struct macho_file_map* fmap, unsigned cmd,
 
     TRACE("(%p/%d, %u, %p, %p)\n", fmap, fmap->fd, cmd, cb, user);
 
-    if ((lc = macho_map_load_commands(fmap)) == MACHO_NO_MAP) return -1;
+    if ((lc = macho_map_load_commands(fmap)) == IMAGE_NO_MAP) return -1;
 
     TRACE("%d total commands\n", fmap->mach_header.ncmds);
 
@@ -376,56 +458,91 @@ static int macho_enum_load_commands(struct macho_file_map* fmap, unsigned cmd,
 }
 
 /******************************************************************
- *              macho_accum_segs_range
+ *              macho_count_sections
+ *
+ * Callback for macho_enum_load_commands.  Counts the number of
+ * significant sections in a Mach-O file.  All commands are
+ * expected to be of LC_SEGMENT[_64] type.
+ */
+static int macho_count_sections(struct macho_file_map* fmap, const struct load_command* lc, void* user)
+{
+    const macho_segment_command* sc = (const macho_segment_command*)lc;
+
+    TRACE("(%p/%d, %p, %p) segment %s\n", fmap, fmap->fd, lc, user, debugstr_an(sc->segname, sizeof(sc->segname)));
+
+    fmap->num_sections += sc->nsects;
+    return 0;
+}
+
+/******************************************************************
+ *              macho_load_section_info
  *
  * Callback for macho_enum_load_commands.  Accumulates the address
- * range covered by the segments of a Mach-O file.  All commands
- * are expected to be of LC_SEGMENT type.
+ * range covered by the segments of a Mach-O file and builds the
+ * section map.  All commands are expected to be of LC_SEGMENT[_64] type.
  */
-static int macho_accum_segs_range(struct macho_file_map* fmap,
-                                  const struct load_command* lc, void* user)
+static int macho_load_section_info(struct macho_file_map* fmap, const struct load_command* lc, void* user)
 {
-    const struct segment_command*   sc = (const struct segment_command*)lc;
-    unsigned                        tmp, page_mask = sysconf( _SC_PAGESIZE ) - 1;
+    const macho_segment_command*    sc = (const macho_segment_command*)lc;
+    int*                            section_index = (int*)user;
+    const macho_section*            section;
+    int                             i;
+    unsigned long                   tmp, page_mask = sysconf( _SC_PAGESIZE ) - 1;
 
-    TRACE("(%p/%d, %p, %p) before: 0x%08x - 0x%08x\n", fmap, fmap->fd, lc, user,
-            (unsigned)fmap->segs_start, (unsigned)fmap->segs_size);
-    TRACE("Segment command vm: 0x%08x - 0x%08x\n", (unsigned)sc->vmaddr,
-            (unsigned)sc->vmaddr + sc->vmsize);
+    TRACE("(%p/%d, %p, %p) before: 0x%08lx - 0x%08lx\n", fmap, fmap->fd, lc, user,
+            (unsigned long)fmap->segs_start, (unsigned long)fmap->segs_size);
+    TRACE("Segment command vm: 0x%08lx - 0x%08lx\n", (unsigned long)sc->vmaddr,
+            (unsigned long)(sc->vmaddr + sc->vmsize));
 
     if (!strncmp(sc->segname, "WINE_", 5))
-    {
         TRACE("Ignoring special Wine segment %s\n", debugstr_an(sc->segname, sizeof(sc->segname)));
-        return 0;
-    }
-    if (!strncmp(sc->segname, "__PAGEZERO", 10))
-    {
+    else if (!strncmp(sc->segname, "__PAGEZERO", 10))
         TRACE("Ignoring __PAGEZERO segment\n");
-        return 0;
+    else
+    {
+        /* If this segment starts before previously-known earliest, record new earliest. */
+        if (sc->vmaddr < fmap->segs_start)
+            fmap->segs_start = sc->vmaddr;
+
+        /* If this segment extends beyond previously-known furthest, record new furthest. */
+        tmp = (sc->vmaddr + sc->vmsize + page_mask) & ~page_mask;
+        if (fmap->segs_size < tmp) fmap->segs_size = tmp;
+
+        TRACE("after: 0x%08lx - 0x%08lx\n", (unsigned long)fmap->segs_start, (unsigned long)fmap->segs_size);
     }
 
-    /* If this segment starts before previously-known earliest, record
-     * new earliest. */
-    if (sc->vmaddr < fmap->segs_start)
-        fmap->segs_start = sc->vmaddr;
-
-    /* If this segment extends beyond previously-known furthest, record
-     * new furthest. */
-    tmp = (sc->vmaddr + sc->vmsize + page_mask) & ~page_mask;
-    if (fmap->segs_size < tmp) fmap->segs_size = tmp;
-
-    TRACE("after: 0x%08x - 0x%08x\n", (unsigned)fmap->segs_start, (unsigned)fmap->segs_size);
+    section = (const macho_section*)(sc + 1);
+    for (i = 0; i < sc->nsects; i++)
+    {
+        fmap->sect[*section_index].section = &section[i];
+        fmap->sect[*section_index].mapped = IMAGE_NO_MAP;
+        (*section_index)++;
+    }
 
     return 0;
 }
 
+/******************************************************************
+ *              reset_file_map
+ */
+static inline void reset_file_map(struct image_file_map* ifm)
+{
+    struct macho_file_map* fmap = &ifm->u.macho;
+
+    fmap->fd = -1;
+    fmap->load_commands = IMAGE_NO_MAP;
+    fmap->num_sections = 0;
+    fmap->sect = NULL;
+}
+
 /******************************************************************
  *              macho_map_file
  *
  * Maps a Mach-O file into memory (and checks it's a real Mach-O file)
  */
-static BOOL macho_map_file(const WCHAR* filenameW, struct macho_file_map* fmap)
+static BOOL macho_map_file(const WCHAR* filenameW, struct image_file_map* ifm)
 {
+    struct macho_file_map* fmap = &ifm->u.macho;
     struct fat_header   fat_header;
     struct stat         statbuf;
     int                 i;
@@ -435,9 +552,14 @@ static BOOL macho_map_file(const WCHAR* filenameW, struct macho_file_map* fmap)
 
     TRACE("(%s, %p)\n", debugstr_w(filenameW), fmap);
 
-    fmap->fd = -1;
-    fmap->load_commands = MACHO_NO_MAP;
-    RtlInitializeBitMap(&fmap->sect_is_code, fmap->sect_is_code_buff, MAX_SECT + 1);
+    reset_file_map(ifm);
+
+    ifm->modtype = DMT_MACHO;
+#ifdef _WIN64
+    ifm->addr_size = 64;
+#else
+    ifm->addr_size = 32;
+#endif
 
     len = WideCharToMultiByte(CP_UNIXCP, 0, filenameW, -1, NULL, 0, NULL, NULL);
     if (!(filename = HeapAlloc(GetProcessHeap(), 0, len)))
@@ -477,20 +599,18 @@ static BOOL macho_map_file(const WCHAR* filenameW, struct macho_file_map* fmap)
             struct fat_arch fat_arch;
             if (read(fmap->fd, &fat_arch, sizeof(fat_arch)) != sizeof(fat_arch))
                 goto done;
-            if (swap_ulong_be_to_host(fat_arch.cputype) == CPU_TYPE_X86)
+            if (swap_ulong_be_to_host(fat_arch.cputype) == TARGET_CPU_TYPE)
             {
                 fmap->arch_offset = swap_ulong_be_to_host(fat_arch.offset);
-                fmap->arch_size = swap_ulong_be_to_host(fat_arch.size);
                 break;
             }
         }
         if (i >= narch) goto done;
-        TRACE("... found x86 arch\n");
+        TRACE("... found target arch (%d)\n", TARGET_CPU_TYPE);
     }
     else
     {
         fmap->arch_offset = 0;
-        fmap->arch_size = statbuf.st_size;
         TRACE("... not a fat header\n");
     }
 
@@ -500,8 +620,8 @@ static BOOL macho_map_file(const WCHAR* filenameW, struct macho_file_map* fmap)
         goto done;
     TRACE("... got possible Mach header\n");
     /* and check for a Mach-O header */
-    if (fmap->mach_header.magic != MH_MAGIC ||
-        fmap->mach_header.cputype != CPU_TYPE_X86) goto done;
+    if (fmap->mach_header.magic != TARGET_MH_MAGIC ||
+        fmap->mach_header.cputype != TARGET_CPU_TYPE) goto done;
     /* Make sure the file type is one of the ones we expect. */
     switch (fmap->mach_header.filetype)
     {
@@ -513,22 +633,35 @@ static BOOL macho_map_file(const WCHAR* filenameW, struct macho_file_map* fmap)
         default:
             goto done;
     }
-    TRACE("... verified Mach x86 header\n");
+    TRACE("... verified Mach header\n");
+
+    fmap->num_sections = 0;
+    if (macho_enum_load_commands(fmap, TARGET_SEGMENT_COMMAND, macho_count_sections, NULL) < 0)
+        goto done;
+    TRACE("%d sections\n", fmap->num_sections);
+
+    fmap->sect = HeapAlloc(GetProcessHeap(), 0, fmap->num_sections * sizeof(fmap->sect[0]));
+    if (!fmap->sect)
+        goto done;
 
     fmap->segs_size = 0;
     fmap->segs_start = ~0L;
 
-    if (macho_enum_load_commands(fmap, LC_SEGMENT, macho_accum_segs_range, NULL) < 0)
+    i = 0;
+    if (macho_enum_load_commands(fmap, TARGET_SEGMENT_COMMAND, macho_load_section_info, &i) < 0)
+    {
+        fmap->num_sections = 0;
         goto done;
+    }
 
     fmap->segs_size -= fmap->segs_start;
-    TRACE("segs_start: 0x%08x, segs_size: 0x%08x\n", (unsigned)fmap->segs_start,
-            (unsigned)fmap->segs_size);
+    TRACE("segs_start: 0x%08lx, segs_size: 0x%08lx\n", (unsigned long)fmap->segs_start,
+            (unsigned long)fmap->segs_size);
 
     ret = TRUE;
 done:
     if (!ret)
-        macho_unmap_file(fmap);
+        macho_unmap_file(ifm);
     HeapFree(GetProcessHeap(), 0, filename);
     return ret;
 }
@@ -538,53 +671,24 @@ done:
  *
  * Unmaps a Mach-O file from memory (previously mapped with macho_map_file)
  */
-static void macho_unmap_file(struct macho_file_map* fmap)
+static void macho_unmap_file(struct image_file_map* ifm)
 {
-    TRACE("(%p/%d)\n", fmap, fmap->fd);
-    if (fmap->fd != -1)
+    TRACE("(%p/%d)\n", ifm, ifm->u.macho.fd);
+    if (ifm->u.macho.fd != -1)
     {
-        macho_unmap_load_commands(fmap);
-        close(fmap->fd);
-        fmap->fd = -1;
+        struct image_section_map ism;
+
+        ism.fmap = ifm;
+        for (ism.sidx = 0; ism.sidx < ifm->u.macho.num_sections; ism.sidx++)
+            macho_unmap_section(&ism);
+
+        HeapFree(GetProcessHeap(), 0, ifm->u.macho.sect);
+        macho_unmap_load_commands(&ifm->u.macho);
+        close(ifm->u.macho.fd);
+        ifm->u.macho.fd = -1;
     }
 }
 
-/******************************************************************
- *              macho_fill_sect_is_code
- *
- * Callback for macho_enum_load_commands.  Determines which segments
- * of a Mach-O file contain code.  All commands are expected to be
- * of LC_SEGMENT type.
- */
-static int macho_fill_sect_is_code(struct macho_file_map* fmap,
-                                   const struct load_command* lc, void* user)
-{
-    const struct segment_command*   sc = (const struct segment_command*)lc;
-    const struct section*           sections;
-    int*                            cursect = user;
-    int                             i;
-
-    TRACE("(%p/%d, %p, %p/%d) scanning %u sections\n", fmap, fmap->fd, lc,
-            cursect, *cursect, sc->nsects);
-
-    sections = (const struct section*)(sc + 1);
-    for (i = 0; i < sc->nsects; i++)
-    {
-        if (*cursect > MAX_SECT) return -1;
-        (*cursect)++;
-
-        if (!(sections[i].flags & SECTION_TYPE) &&
-            (sections[i].flags & (S_ATTR_PURE_INSTRUCTIONS|S_ATTR_SOME_INSTRUCTIONS)))
-            RtlSetBits(&fmap->sect_is_code, *cursect, 1);
-        else
-            RtlClearBits(&fmap->sect_is_code, *cursect, 1);
-        TRACE("Section %d (%d of this segment) is%s code\n", *cursect, i,
-                (RtlAreBitsSet(&fmap->sect_is_code, *cursect, 1) ? "" : " not"));
-    }
-
-    return 0;
-}
-
 /******************************************************************
  *              macho_sect_is_code
  *
@@ -594,17 +698,19 @@ static int macho_fill_sect_is_code(struct macho_file_map* fmap,
  */
 static BOOL macho_sect_is_code(struct macho_file_map* fmap, unsigned char sectidx)
 {
+    BOOL ret;
+
     TRACE("(%p/%d, %u)\n", fmap, fmap->fd, sectidx);
 
-    if (!RtlAreBitsSet(&fmap->sect_is_code, 0, 1))
-    {
-        int cursect = 0;
-        if (macho_enum_load_commands(fmap, LC_SEGMENT, macho_fill_sect_is_code, &cursect) < 0)
-            WARN("Couldn't load sect_is_code map\n");
-        RtlSetBits(&fmap->sect_is_code, 0, 1);
-    }
+    if (!sectidx) return FALSE;
 
-    return RtlAreBitsSet(&fmap->sect_is_code, sectidx, 1);
+    sectidx--; /* convert from 1-based to 0-based */
+    if (sectidx >= fmap->num_sections) return FALSE;
+
+    ret = (!(fmap->sect[sectidx].section->flags & SECTION_TYPE) &&
+           (fmap->sect[sectidx].section->flags & (S_ATTR_PURE_INSTRUCTIONS|S_ATTR_SOME_INSTRUCTIONS)));
+    TRACE("-> %d\n", ret);
+    return ret;
 }
 
 struct symtab_elt
@@ -667,24 +773,24 @@ static int macho_parse_symtab(struct macho_file_map* fmap,
 {
     const struct symtab_command*    sc = (const struct symtab_command*)lc;
     struct macho_debug_info*        mdi = user;
-    const struct nlist*             stab;
+    const macho_nlist*              stab;
     const char*                     stabstr;
     int                             ret = 0;
 
     TRACE("(%p/%d, %p, %p) %u syms at 0x%08x, strings 0x%08x - 0x%08x\n", fmap, fmap->fd, lc,
             user, sc->nsyms, sc->symoff, sc->stroff, sc->stroff + sc->strsize);
 
-    if (!macho_map_ranges(fmap, sc->symoff, sc->nsyms * sizeof(struct nlist),
+    if (!macho_map_ranges(fmap, sc->symoff, sc->nsyms * sizeof(macho_nlist),
             sc->stroff, sc->strsize, (const void**)&stab, (const void**)&stabstr))
         return 0;
 
     if (!stabs_parse(mdi->module,
                      mdi->module->format_info[DFI_MACHO]->u.macho_info->load_addr - fmap->segs_start,
-                     stab, sc->nsyms * sizeof(struct nlist),
+                     stab, sc->nsyms * sizeof(macho_nlist),
                      stabstr, sc->strsize, macho_stabs_def_cb, mdi))
         ret = -1;
 
-    macho_unmap_ranges(fmap, sc->symoff, sc->nsyms * sizeof(struct nlist),
+    macho_unmap_ranges(fmap, sc->symoff, sc->nsyms * sizeof(macho_nlist),
             sc->stroff, sc->strsize, (const void**)&stab, (const void**)&stabstr);
 
     return ret;
@@ -861,20 +967,24 @@ static void macho_finish_stabs(struct module* module, struct hash_table* ht_symt
 }
 
 /******************************************************************
- *              macho_load_debug_info_from_map
+ *              macho_load_debug_info
  *
- * Loads the symbolic information from a Mach-O module.
- * Returns
- *      FALSE if the file doesn't contain symbolic info (or this info
- *              cannot be read or parsed)
- *      TRUE on success
+ * Loads Mach-O debugging information from the module image file.
  */
-static BOOL macho_load_debug_info_from_map(struct module* module,
-                                           struct macho_file_map* fmap)
+BOOL macho_load_debug_info(struct module* module)
 {
     BOOL                    ret = FALSE;
     struct macho_debug_info mdi;
     int                     result;
+    struct macho_file_map  *fmap;
+
+    if (module->type != DMT_MACHO || !module->format_info[DFI_MACHO]->u.macho_info)
+    {
+        ERR("Bad Mach-O module '%s'\n", debugstr_w(module->module.LoadedImageName));
+        return FALSE;
+    }
+
+    fmap = &module->format_info[DFI_MACHO]->u.macho_info->file_map.u.macho;
 
     TRACE("(%p, %p/%d)\n", module, fmap, fmap->fd);
 
@@ -892,40 +1002,14 @@ static BOOL macho_load_debug_info_from_map(struct module* module,
 
     macho_finish_stabs(module, &mdi.ht_symtab);
 
+    if (dwarf2_parse(module, module->reloc_delta, NULL /* FIXME: some thunks to deal with ? */,
+                     &module->format_info[DFI_MACHO]->u.macho_info->file_map))
+        ret = TRUE;
+
     pool_destroy(&mdi.pool);
     return ret;
 }
 
-/******************************************************************
- *              macho_load_debug_info
- *
- * Loads Mach-O debugging information from the module image file.
- */
-BOOL macho_load_debug_info(struct module* module, struct macho_file_map* fmap)
-{
-    BOOL                    ret = TRUE;
-    struct macho_file_map   my_fmap;
-
-    TRACE("(%p, %p/%d)\n", module, fmap, fmap ? fmap->fd : -1);
-
-    if (module->type != DMT_MACHO || !module->format_info[DFI_MACHO]->u.macho_info)
-    {
-        ERR("Bad Mach-O module '%s'\n", debugstr_w(module->module.LoadedImageName));
-        return FALSE;
-    }
-
-    if (!fmap)
-    {
-        fmap = &my_fmap;
-        ret = macho_map_file(module->module.LoadedImageName, fmap);
-    }
-    if (ret)
-        ret = macho_load_debug_info_from_map(module, fmap);
-
-    if (fmap == &my_fmap) macho_unmap_file(fmap);
-    return ret;
-}
-
 /******************************************************************
  *              macho_fetch_file_info
  *
@@ -934,18 +1018,27 @@ BOOL macho_load_debug_info(struct module* module, struct macho_file_map* fmap)
 BOOL macho_fetch_file_info(const WCHAR* name, DWORD_PTR* base,
                            DWORD* size, DWORD* checksum)
 {
-    struct macho_file_map fmap;
+    struct image_file_map fmap;
 
     TRACE("(%s, %p, %p, %p)\n", debugstr_w(name), base, size, checksum);
 
     if (!macho_map_file(name, &fmap)) return FALSE;
-    if (base) *base = fmap.segs_start;
-    *size = fmap.segs_size;
-    *checksum = calc_crc32(fmap.fd);
+    if (base) *base = fmap.u.macho.segs_start;
+    *size = fmap.u.macho.segs_size;
+    *checksum = calc_crc32(fmap.u.macho.fd);
     macho_unmap_file(&fmap);
     return TRUE;
 }
 
+/******************************************************************
+ *              macho_module_remove
+ */
+static void macho_module_remove(struct process* pcs, struct module_format* modfmt)
+{
+    macho_unmap_file(&modfmt->u.macho_info->file_map);
+    HeapFree(GetProcessHeap(), 0, modfmt);
+}
+
 /******************************************************************
  *              macho_load_file
  *
@@ -960,7 +1053,7 @@ static BOOL macho_load_file(struct process* pcs, const WCHAR* filename,
                             unsigned long load_addr, struct macho_info* macho_info)
 {
     BOOL                    ret = TRUE;
-    struct macho_file_map   fmap;
+    struct image_file_map   fmap;
 
     TRACE("(%p/%p, %s, 0x%08lx, %p/0x%08x)\n", pcs, pcs->handle, debugstr_w(filename),
             load_addr, macho_info, macho_info->flags);
@@ -981,19 +1074,20 @@ static BOOL macho_load_file(struct process* pcs, const WCHAR* filename,
                                            &pbi, sizeof(pbi), NULL);
         if (status == STATUS_SUCCESS)
         {
-            ULONG dyld_image_info;
+            ULONG_PTR dyld_image_info;
 
             /* Read dyld image info address from PEB */
-            if (ReadProcessMemory(pcs->handle, &pbi.PebBaseAddress->Reserved,
+            if (ReadProcessMemory(pcs->handle, &pbi.PebBaseAddress->Reserved[0],
                                   &dyld_image_info, sizeof(dyld_image_info), NULL))
             {
-                TRACE("got dyld_image_info 0x%08x from PEB %p MacDyldImageInfo %p\n",
-                      dyld_image_info, pbi.PebBaseAddress, &pbi.PebBaseAddress->Reserved);
+                TRACE("got dyld_image_info 0x%08lx from PEB %p MacDyldImageInfo %p\n",
+                      (unsigned long)dyld_image_info, pbi.PebBaseAddress, &pbi.PebBaseAddress->Reserved);
                 macho_info->dbg_hdr_addr = dyld_image_info;
                 ret = TRUE;
             }
         }
 
+#ifndef __LP64__ /* No reading the symtab with nlist(3) in LP64 */
         if (!ret)
         {
             static void* dyld_all_image_infos_addr;
@@ -1017,6 +1111,7 @@ static BOOL macho_load_file(struct process* pcs, const WCHAR* filename,
                 ret = TRUE;
             }
         }
+#endif
     }
 
     if (macho_info->flags & MACHO_INFO_MODULE)
@@ -1026,27 +1121,30 @@ static BOOL macho_load_file(struct process* pcs, const WCHAR* filename,
             HeapAlloc(GetProcessHeap(), 0, sizeof(struct module_format) + sizeof(struct macho_module_info));
         if (!modfmt) goto leave;
         if (!load_addr)
-            load_addr = fmap.segs_start;
+            load_addr = fmap.u.macho.segs_start;
         macho_info->module = module_new(pcs, filename, DMT_MACHO, FALSE, load_addr,
-                                        fmap.segs_size, 0, calc_crc32(fmap.fd));
+                                        fmap.u.macho.segs_size, 0, calc_crc32(fmap.u.macho.fd));
         if (!macho_info->module)
         {
             HeapFree(GetProcessHeap(), 0, modfmt);
             goto leave;
         }
+        macho_info->module->reloc_delta = macho_info->module->module.BaseOfImage - fmap.u.macho.segs_start;
         macho_module_info = (void*)(modfmt + 1);
         macho_info->module->format_info[DFI_MACHO] = modfmt;
 
         modfmt->module       = macho_info->module;
-        modfmt->remove       = NULL;
+        modfmt->remove       = macho_module_remove;
         modfmt->loc_compute  = NULL;
         modfmt->u.macho_info = macho_module_info;
 
         macho_module_info->load_addr = load_addr;
 
+        macho_module_info->file_map = fmap;
+        reset_file_map(&fmap);
         if (dbghelp_options & SYMOPT_DEFERRED_LOADS)
             macho_info->module->module.SymType = SymDeferred;
-        else if (!macho_load_debug_info(macho_info->module, &fmap))
+        else if (!macho_load_debug_info(macho_info->module))
             ret = FALSE;
 
         macho_info->module->format_info[DFI_MACHO]->u.macho_info->in_use = 1;
@@ -1452,6 +1550,30 @@ struct module*  macho_load_module(struct process* pcs, const WCHAR* name, unsign
 
 #else  /* HAVE_MACH_O_LOADER_H */
 
+BOOL macho_find_section(struct image_file_map* ifm, const char* segname, const char* sectname, struct image_section_map* ism)
+{
+    return FALSE;
+}
+
+const char* macho_map_section(struct image_section_map* ism)
+{
+    return NULL;
+}
+
+void macho_unmap_section(struct image_section_map* ism)
+{
+}
+
+DWORD_PTR macho_get_map_rva(const struct image_section_map* ism)
+{
+    return 0;
+}
+
+unsigned macho_get_map_size(const struct image_section_map* ism)
+{
+    return 0;
+}
+
 BOOL    macho_synchronize_module_list(struct process* pcs)
 {
     return FALSE;
@@ -1478,7 +1600,7 @@ struct module*  macho_load_module(struct process* pcs, const WCHAR* name, unsign
     return NULL;
 }
 
-BOOL macho_load_debug_info(struct module* module, struct macho_file_map* fmap)
+BOOL macho_load_debug_info(struct module* module)
 {
     return FALSE;
 }
