@@ -256,6 +256,10 @@ static void release_inner_window(HTMLInnerWindow *This)
         IHTMLOptionElementFactory_Release(&This->option_factory->IHTMLOptionElementFactory_iface);
     }
 
+    if(This->xml_factory) {
+        IHTMLXMLHttpRequestFactory_Release(&This->xml_factory->IHTMLXMLHttpRequestFactory_iface);
+    }
+
     if(This->screen)
         IHTMLScreen_Release(This->screen);
 
@@ -406,6 +410,11 @@ HRESULT get_frame_by_name(HTMLOuterWindow *This, const WCHAR *name, BOOL deep, H
 
         nsIDOMWindow_Release(nswindow);
 
+        if(!window_iter) {
+            WARN("nsIDOMWindow without HTMLOuterWindow: %p\n", nswindow);
+            continue;
+        }
+
         hres = IHTMLElement_get_id(&window_iter->frame_element->element.IHTMLElement_iface, &id);
         if(FAILED(hres)) {
             FIXME("IHTMLElement_get_id failed: 0x%08x\n", hres);
@@ -1960,8 +1969,23 @@ static HRESULT WINAPI HTMLWindow5_put_XMLHttpRequest(IHTMLWindow5 *iface, VARIAN
 static HRESULT WINAPI HTMLWindow5_get_XMLHttpRequest(IHTMLWindow5 *iface, VARIANT *p)
 {
     HTMLWindow *This = impl_from_IHTMLWindow5(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    HTMLInnerWindow *window = This->inner_window;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    if(!window->xml_factory) {
+        HRESULT hres;
+
+        hres = HTMLXMLHttpRequestFactory_Create(window, &window->xml_factory);
+        if(FAILED(hres))
+            return hres;
+    }
+
+    V_VT(p) = VT_DISPATCH;
+    V_DISPATCH(p) = (IDispatch*)&window->xml_factory->IHTMLXMLHttpRequestFactory_iface;
+    IDispatch_AddRef(V_DISPATCH(p));
+
+    return S_OK;
 }
 
 static const IHTMLWindow5Vtbl HTMLWindow5Vtbl = {
