@@ -533,6 +533,15 @@ SIZE_T WINAPI RtlCompareMemoryUlong(const ULONG *Source1, SIZE_T Length, ULONG d
     return i * sizeof(ULONG);
 }
 
+/******************************************************************************
+ *  RtlCopyMemory   [NTDLL.@]
+ */
+#undef RtlCopyMemory
+void WINAPI RtlCopyMemory(void *dest, const void *src, SIZE_T len)
+{
+    memcpy(dest, src, len);
+}
+
 /******************************************************************************
  *  RtlAssert                           [NTDLL.@]
  *
